{
    "content" : {
        "with": [
            {
                "some": {
                    "deep": [
                        {
                            "nesting": {
                                "MODULE": "MyApp::MyAction",
                                "replace": "me"
                            }
                        }
                    ]
                }
            }
        ]
    }
}
